/************************************************************
JavaScript Encapsulation Library.

Copyright (C) 2002, Alben Software, Inc.
All rights reserved.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

Alben Software, Inc.
Benny Pollak, President.
20 E 9th Street.
NY, NY 10003
benny@alben.com
*********************************************************************/
#include "stdafx.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include  <strstream>

#include "jsstr.h"
#include "js.h"

static JSBool addthem(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
		      jsval *rval)
{
	double *x = JSVAL_TO_DOUBLE(argv[0]);
	double *y = JSVAL_TO_DOUBLE(argv[1]);//
	*rval = DOUBLE_TO_JSVAL(JS_NewDouble(cx, *x + *y));
	return JS_TRUE;
}
// Function to be added to the JS interpreter
static JSBool GetPassword(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
		      jsval *rval)
{
	const char *result = "hello world";
	*rval = STRING_TO_JSVAL(JS_NewString(cx, JS_strdup(cx, result), strlen(result)));
	return JS_TRUE;
}
int JS::LookupProperty(char *name, jsval &xv)
{
	return JS_LookupProperty(cx_, glob_, name, &xv) && !JSVAL_IS_VOID(xv);
}
void JS::MyErrorReporter(JSContext *cx, const char *message,
                                    JSErrorReport *report)
{
	if (err_) {
		free(err_);
		err_ = NULL;
	}
	std::strstream str;
	str << "'" << message << '\'' << std::endl
		<< "Line: " << report->lineno << std::endl
		<< ">>>> " << (report->tokenptr?report->tokenptr:"") << std::ends;

	err_ = strdup(str.str());
	delete str.str();
}
char *JS::getError()
{
	return err_;
}
void JS::tester()
{
    jsval argv[2], rval;

	double x = 3.0;
	double y = 5.0;//
    //argv[0] = DOUBLE_TO_JSVAL(JS_NewDouble(cx_, x));
    argv[0] = DOUBLE_TO_JSVAL(JS_NewDouble(cx_, x));
    argv[1] = DOUBLE_TO_JSVAL(JS_NewDouble(cx_, y));
    JSBool ok = JS_CallFunctionName(cx_, glob_, "addthem", 2, argv, &rval);
	double *z = JSVAL_TO_DOUBLE(rval);
    ok = JS_CallFunctionName(cx_, glob_, "GetPassword", 0, argv, &rval);


	JSString *s = JSVAL_TO_STRING(rval);
	s->length;
	s->chars;
	char *str = JS_GetStringBytes(s);
//	JSString *s = JS_ValueToString(cx, rval);
    char *script = "(function fact(n){if (n <= 1) return 1; return n * fact(n-1)})(5)"; 
    JSString *jstr; 

    ok = JS_EvaluateScript(cx_, glob_, script, strlen(script), 
                           "zz.js", 5, &rval); 
    jstr = JS_ValueToString(cx_, rval); 
    str = JS_GetStringBytes(jstr);

	script = "function xmultthem(a,b){return a * b + 1}"; 
	//JSScript *fun = 
	ok = JS_EvaluateScript(cx_, glob_,
		script, strlen(script),
		"", 0, &rval);
	if (!ok) {
	    jstr = JS_ValueToString(cx_, rval); 
		str = JS_GetStringBytes(jstr);
	}

    argv[0] = DOUBLE_TO_JSVAL(JS_NewDouble(cx_, x));
    argv[1] = DOUBLE_TO_JSVAL(JS_NewDouble(cx_, y));
    ok = JS_CallFunctionName(cx_, glob_, "xmultthem", 2, argv, &rval);
	z = JSVAL_TO_DOUBLE(rval);
}
int JS::InstallScript(const char *script)
{
	jsval rval;
	JSBool ok = JS_EvaluateScript(cx_, glob_,
		script, strlen(script),
		"", 1, &rval);
	return ok;
}
JS::JS() : cx_(0), rt_(0), glob_(0), initialized_(0)
{
}
int JS::init()
{
	if (initialized_) return cx_ != NULL && glob_ != NULL;
	initialized_ = 1;
	static JSClass clasp;
	/* initialize the JS run time, and return result in rt */
	rt_ = JS_NewRuntime(8L * 1024L * 1024L);

	/* if rt does not have a value, end the program here */
	if (!rt_)
		return 0;

	/* create a context and associate it with the JS run time */
	cx_ = JS_NewContext(rt_, 8192);

	/* if cx does not have a value, end the program here */
	if (cx_ == NULL)
		return 0;

	static JSClass global_class = { 
        "global",0, 
        JS_PropertyStub,JS_PropertyStub,JS_PropertyStub,JS_PropertyStub, 
        JS_EnumerateStub,JS_ResolveStub,JS_ConvertStub,JS_FinalizeStub 
    }; 	/* create the global object here */
	glob_ = JS_NewObject(cx_, &global_class, NULL, NULL);

	/* initialize the built-in JS objects and the global object */
	builtins_ = JS_InitStandardClasses(cx_, glob_);
	int status = ExtendJS();
	JS_SetErrorReporter(cx_, JS::MyErrorReporter);

	// Testing...
	tester();

	
	return 1;
}
extern "C" {
	void main(){}
};
JS::~JS()
{
	/* Before exiting the application, free the JS run time */
    /* For each context you've created: */
#if 1
//free(JS_getit());
    if (cx_) {
		JS_DestroyContext(cx_);
	}
	if (rt_) {
		/* And finally: */
		//JS_Finish(rt_);
		JS_DestroyRuntime(rt_);
	}
#endif

}
// Function to be added to the JS interpreter
/* Use a JSFunctionSpec array terminated with a null name to define a
 * bunch of native functions.
 */
static JSFunctionSpec my_functions[] = {
/*    name				native			nargs    */
    {"addthem",			addthem,		2},
    {"GetPassword",   GetPassword,	0},
    {0}
};

typedef struct {
		char *varname;
		char *initval;
} jsinitvars_t;

jsinitvars_t jsinitvars[] = {
	{"dataclar_info", "-"},
	{"page_to_remove", "-1"},
	{"doc_path", "-"},
	{"initialized", "-"},
	{"allow_dclars", "0"},
	{"inet_data_clarifications", ""},
	{"template_list", ""},
	{"audit", "-"},
	{"inet_change_audit", ""},
	{"cgi_vars", ""},
	{"back_ip", ""},
	{"back_port", ""},
	{"back_cgi", ""},
	{"action_kludge", ""},
	{"debug", ""},
	{NULL, NULL},
};
int JS::ExtendJS()
{
	//static int is_initialized = 0;if (is_initialized) return 1;is_initialized = 1;
	char buf[4096];
	
	// Add a function to the JS interpreter

/*
    jsval argv[2], rval;
    argv[0] = 1;
    argv[1] = 2;
    JSBool ok = JS_CallFunctionName(cx_, inet, "addthem", 2, argv, &rval);
*/

	if (!JS_DefineFunctions(cx_, glob_, my_functions)) {
		return 0;
	}

	jsval	xv;
	if (!JS_LookupProperty(cx_, glob_, "zaddthem", &xv)
				|| JSVAL_IS_VOID(xv)) {
		sprintf(buf, "Could not lookup property '%s'", "addthem");
		//return 0;
	}
	
	/*
		Create INet variables
	for (jsinitvars_t *jsinitvarsp = jsinitvars; jsinitvarsp->varname; jsinitvarsp++) {
		SetStringVariable(jsinitvarsp->varname, jsinitvarsp->initval);
	}
	*/
	if (!JS_LookupProperty(cx_, glob_, "zaddthem", &xv)
				|| JSVAL_IS_VOID(xv)) {
		sprintf(buf, "Could not lookup property '%s'", "addthem");
		return 0;
	}
	
	return 1;
}

void JS::SetStringVariable(const char *name, const char *val)
{
	//char buf[4096];
	// Add a property

	jsval vp = STRING_TO_JSVAL(JS_NewString(cx_, JS_strdup(cx_, val), strlen(val)));
	if (!JS_SetProperty(cx_, glob_, name, &vp)) {
		//sprintf(buf, "Could not add property '%s'", name);
		return;
	}
}
char *JS::GetStringVariable(char *name)
{
	//char buf[4096];
	// Add a property
	jsval	jv;
	if (!JS_LookupProperty(cx_, glob_, name, &jv)) {
		return "lookup";
	}
	if (JSVAL_IS_VOID(jv) || JSVAL_IS_NULL(jv) || !JSVAL_IS_STRING(jv)) {
		return "bad";
	}
	return JS_GetStringBytes(JSVAL_TO_STRING(jv));

}
int JS::Call(const char *func_name, uintN argc, jsval *argv, jsval *prval)
{
	jsval	xv;
	char buf[512];
	if (!JS_LookupProperty(cx_, glob_, func_name, &xv)
				|| JSVAL_IS_VOID(xv)) {
		sprintf(buf, "Could not find function '%s'", func_name);
		return 0;
	}

	JSBool ok = JS_CallFunctionName(cx_, glob_, func_name, argc, argv, prval);
	return ok;
}
void JS::toJSVal(long var, jsval &ret)
{
	ret = DOUBLE_TO_JSVAL(JS_NewDouble(cx_, var));
}
void JS::toJSVal(long *vars, int count, jsval &ret)
{
	JSObject *obj = JS_NewArrayObject(cx_, 0, NULL);
	ret = OBJECT_TO_JSVAL(obj);
	for (int i = 0; i < count; i++) {
		jsval d;
		JS_NewDoubleValue(cx_, (double)vars[i], &d);
		JSBool ok = JS_SetElement(cx_, obj, i, &d);
	}
}
void JS::toJSVal(short *vars, int count, jsval &ret)
{
	JSObject *obj = JS_NewArrayObject(cx_, 0, NULL);
	ret = OBJECT_TO_JSVAL(obj);
	for (int i = 0; i < count; i++) {
		jsval d;
		JS_NewDoubleValue(cx_, (double)vars[i], &d);
		JSBool ok = JS_SetElement(cx_, obj, i, &d);
	}
}
void JS::toJSVal(unsigned short var, jsval &ret)
{
	ret = DOUBLE_TO_JSVAL(JS_NewDouble(cx_, var));
}
void JS::toJSVal(short var, jsval &ret)
{
	ret = DOUBLE_TO_JSVAL(JS_NewDouble(cx_, var));
}
void JS::toJSVal(float var, jsval &ret)
{
	ret = DOUBLE_TO_JSVAL(JS_NewDouble(cx_, var));
}
void JS::toJSVal(double var, jsval &ret)
{
	double xx = 1.7976931348623158e+308;
	jsdouble *dxx = JS_NewDouble(cx_, xx);
	jsval jxx = DOUBLE_TO_JSVAL(dxx);
	long jxa[2];
	jxa[0] = ((long*)jxx)[0];
	jxa[1] = ((long*)jxx)[1];
	double * xx1 = JSVAL_TO_DOUBLE(jxx);
	long jxa1[2];
	jxa1[0] = ((long*)xx1)[0];
	jxa1[1] = ((long*)xx1)[1];
	long pdxx = *(long*)&dxx;
	long pxx = *(long*)&jxx;
	long pxx1 = *(long*)&xx1;
jsval p;
JS_NewNumberValue(cx_, xx, &p);
	ret = DOUBLE_TO_JSVAL(JS_NewDouble(cx_, var));
}
void JS::toJSVal(double *vars, int count, jsval &ret)
{
	JSObject *obj = JS_NewArrayObject(cx_, 0, NULL);
	ret = OBJECT_TO_JSVAL(obj);
	for (int i = 0; i < count; i++) {
		jsval d;
		JS_NewDoubleValue(cx_, vars[i], &d);
		JSBool ok = JS_SetElement(cx_, obj, i, &d);
	}
}
void JS::toJSVal(float *vars, int count, jsval &ret)
{
	JSObject *obj = JS_NewArrayObject(cx_, 0, NULL);
	ret = OBJECT_TO_JSVAL(obj);
	for (int i = 0; i < count; i++) {
		jsval d;
		JS_NewDoubleValue(cx_, vars[i], &d);
		JSBool ok = JS_SetElement(cx_, obj, i, &d);
	}
}
void JS::toJSVal(unsigned short *var, jsval &ret)
{
	static unsigned short null[] = {'\0', '\0'};
	int len = wcslen(var);
	wchar_t *str = NULL;
	if (1&&var && len > 0) {
		str = (wchar_t*)JS_malloc(cx_, len*2);
		memcpy(str, (char*)var, len*2);
	} else {
		len = 0;
		str = (wchar_t*)JS_malloc(cx_, 2);
		memcpy(str, &null, 2);
	}
	ret = STRING_TO_JSVAL(JS_NewUCString(cx_, str, len));
}
void JS::toJSVal(unsigned short *vars[], int count, jsval &ret)
{
	JSObject *obj = JS_NewArrayObject(cx_, count, NULL);
	for (int i = 0; i < count; i++) {
		jsval d;
		toJSVal(vars[i], d);
		JSBool ok = JS_SetElement(cx_, obj, i, &d);
	}
	ret = OBJECT_TO_JSVAL(obj);
}
void JS::toJSValArray(jsval *vars, int count, jsval &ret)
{
	JSObject *obj = JS_NewArrayObject(cx_, count, NULL);
	for (int i = 0; i < count; i++) {
		//jsval d;
		//toJSVal(vars[i], d);
		JSBool ok = JS_SetElement(cx_, obj, i, &vars[i]);
	}
	ret = OBJECT_TO_JSVAL(obj);
}
void JS::toJSBoolVal(short var, jsval &ret)
{
	ret = BOOLEAN_TO_JSVAL(var);
}
void JS::newJSArray(int count, jsval &ret)
{
	JSObject *obj = JS_NewArrayObject(cx_, count, NULL);
	ret = OBJECT_TO_JSVAL(obj);
}
void JS::addToJSArray(jsval &array, int index, jsval &item)
{
	JSObject *obj = JSVAL_TO_OBJECT(array);
	JSBool ok = JS_SetElement(cx_, obj, index, &item);
}

int JS::isJSBoolVal(const jsval &var)
{
	return JSVAL_IS_BOOLEAN(var);
}
int JS::isJSIntVal(const jsval &var)
{
	return JSVAL_IS_INT(var);
}
int JS::isJSDoubleVal(const jsval &var)
{
	return JSVAL_IS_DOUBLE(var);
}
int JS::isJSStringVal(const jsval &var)
{
	return JSVAL_IS_STRING(var);
}

short JS::getJSBoolVal(const jsval &var)
{
	return JSVAL_TO_BOOLEAN(var);
}
long JS::getJSIntVal(const jsval &var)
{
	return JSVAL_TO_INT(var);
}
double JS::getJSDoubleVal(const jsval &var)
{
	return *JSVAL_TO_DOUBLE(var);
}
char *JS::getJSStringVal(const jsval &var)
{
	return JS_GetStringBytes(JSVAL_TO_STRING(var));
}
char *JS::err_;