/************************************************************
JavaScript Encapsulation Library.

Copyright (C) 2002, Alben Software, Inc.
All rights reserved.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

Alben Software, Inc.
Benny Pollak, President.
20 E 9th Street.
NY, NY 10003
benny@alben.com
*********************************************************************/
// JSCOMUtils.cpp: implementation of the JSCOMUtils class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "JSCOMUtils.h"
#include  <strstream>
//#include "JSScript.h"
//#include "JSHandler.h"
#include "JS.h"
#include <comdef.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

JSCOMUtils::JSCOMUtils()
{

}

JSCOMUtils::~JSCOMUtils()
{

}
#define CASE_ARRAY(type) \
		{ \
		SAFEARRAY *pArray = NULL; \
		if (var.vt & VT_BYREF) { \
			if (var.pparray == NULL) { retcode = 0; break; } \
			pArray = *var.pparray; \
		} else { \
			pArray = var.parray; \
		} \
		type *vars = NULL; \
		HRESULT hr = SafeArrayAccessData(pArray,(void HUGEP**)&vars); \
		unsigned long count = pArray->rgsabound[0].cElements; \
		js_.toJSVal(vars, count, ret); \
		SafeArrayUnaccessData(pArray); \
		} \
		break;

#define CASE_NATIVE(member) \
		if (var.vt & VT_BYREF) { \
			if (var.plVal == NULL) retcode = 0; \
			else js_.toJSVal(*var.p##member, ret); \
		} else { \
			js_.toJSVal(var.member, ret); \
		} \
		break;

int JSCOMUtils::toJSVal(JS &js_, const VARIANT &var, jsval &ret)
{
	if (var.vt == VT_DISPATCH)
	{
		DISPID dispid;
		LPOLESTR funcname = L"Value";
		int result = var.pdispVal->GetIDsOfNames(IID_NULL,&funcname,1,LOCALE_SYSTEM_DEFAULT,&dispid);
		if (result == DISP_E_UNKNOWNNAME) return 0;
		VARIANTARG xv;
		if (result == S_OK)
		{
			DISPPARAMS dispparams = {NULL,NULL,0,0};
			var.pdispVal->Invoke(dispid,IID_NULL,LOCALE_SYSTEM_DEFAULT,DISPATCH_PROPERTYGET,
			&dispparams,&xv,NULL,NULL);
		}
		return toJSVal(js_, xv, ret);
	}	
	int retcode = 1;
	switch (var.vt & (~VT_BYREF)) {
	case VT_I4:
		CASE_NATIVE(lVal)
	case VT_UI1:
		CASE_NATIVE(uiVal)
	case VT_I2:
		CASE_NATIVE(iVal)
	case VT_R4:
		CASE_NATIVE(fltVal)
	case VT_R8:
		CASE_NATIVE(dblVal)
	case VT_BOOL:
		CASE_NATIVE(boolVal)
	case VT_BSTR:
		CASE_NATIVE(bstrVal)
	case VT_ARRAY|VT_I4:
		CASE_ARRAY(long)
	case VT_ARRAY|VT_I2:
		CASE_ARRAY(short)
	case VT_ARRAY|VT_R4:
		CASE_ARRAY(float)
	case VT_ARRAY|VT_R8:
		CASE_ARRAY(double)
	case VT_ARRAY|VT_BSTR:
		CASE_ARRAY(BSTR)
	case VT_VARIANT:
		{
		VARIANTARG xv = *var.pvarVal;
		xv.vt = VT_ARRAY|VT_VARIANT;
		return toJSVal(js_, xv, ret);
		}
		break;
	case VT_ARRAY|VT_VARIANT:
		{
		SAFEARRAY *pArray = NULL;
		if (var.vt & VT_BYREF) {
			if (var.pparray == NULL) { retcode = 0; break; }
			pArray = *var.pparray;
		} else {
			pArray = var.parray;
		}
		VARIANT *vars = NULL;

		HRESULT hr = SafeArrayAccessData(pArray,(void HUGEP**)&vars);
		int count = 0;
		if (pArray->cDims == 1) {
			count = pArray->rgsabound[0].cElements;
		} else {
			unsigned long lbound = pArray->rgsabound[0].cElements;
			unsigned long ubound = pArray->rgsabound[1].cElements;
			if (ubound < lbound) return 0;
			count = ubound-lbound+1;
		}
		jsval *array = new jsval[count];
		try {
			for (int i = 0; i < count; i++) {
				if (!toJSVal(js_, vars[i], array[i])) {
					throw exception("bad argument in array");
				}
			}
			js_.toJSValArray(array, count, ret);
		} catch (...) {
			retcode = 0;
		}
		delete [] array;
		SafeArrayUnaccessData(pArray);
			/*
			*/
		}
		break;
	case VT_EMPTY:
	case VT_NULL:
		js_.toJSVal((double)0.0, ret);
		break;
	default:
		retcode = 0;
	}
	return retcode;
}
int JSCOMUtils::fromJSVal(JS &js_, const jsval &var, VARIANT &ret)
{
	int retcode = 1;
	if (js_.isJSStringVal(var)) {
		USES_CONVERSION;
		char *s = js_.getJSStringVal(var);
		ret.vt = VT_BSTR;
		ret.bstrVal = ::SysAllocString(A2BSTR(s));
	} else if (js_.isJSIntVal(var)) {
		ret.vt = VT_I4;
		ret.lVal = js_.getJSIntVal(var);
	} else if (js_.isJSDoubleVal(var)) {
		ret.vt = VT_R8;
		ret.dblVal = js_.getJSDoubleVal(var);
	} else if (js_.isJSBoolVal(var)) {
		ret.boolVal = js_.getJSBoolVal(var);
	} else if (JSVAL_IS_NULL(var)) {
		ret.vt = VT_UNKNOWN;
		retcode = 0;
	} else if (JSVAL_IS_NUMBER(var)) {
		ret.vt = VT_UNKNOWN;
		retcode = 0;
	} else if (JSVAL_IS_PRIMITIVE(var)) {
		ret.vt = VT_UNKNOWN;
		retcode = 0;
	} else {
		ret.vt = VT_UNKNOWN;
		retcode = 0;
	}
	return retcode;
}
