/************************************************************
JavaScript Encapsulation Library.

Copyright (C) 2002, Alben Software, Inc.
All rights reserved.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

Alben Software, Inc.
Benny Pollak, President.
20 E 9th Street.
NY, NY 10003
benny@alben.com
*********************************************************************/
// JSHandler.cpp : Implementation of CJSHandler
#include "stdafx.h"
#include  <strstream>
#include "JSScript.h"
#include "JSHandler.h"
#include "JS.h"
#include <comdef.h>
#include "JSCOMUtils.h"
/////////////////////////////////////////////////////////////////////////////
// CJSHandler

STDMETHODIMP CJSHandler::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IJSHandler
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}
CJSHandler::CJSHandler()
{
}
CJSHandler::~CJSHandler()
{
}
HRESULT CJSHandler::FinalConstruct()
{
	if (js_.init()) {
		return S_OK;
	} else {
		wError_.SetError("Cannot initialize JS engine.");
		return E_FAIL;
	}
}
void CJSHandler::FinalRelease()
{
}
int CJSHandler::toJSVal(const VARIANT &var, jsval &ret)
{
	return JSCOMUtils::toJSVal(js_, var, ret);
}
int CJSHandler::fromJSVal(const jsval &var, VARIANT &ret)
{
	return JSCOMUtils::fromJSVal(js_, var, ret);
}
HRESULT CJSHandler::Call(BSTR Func, VARIANT *Ret, VARIANT *Arg, ...)
{
    jsval rval;
	va_list		marker;
	jsval		argv[50];
	int			argc = 0;
	va_start( marker, Arg );     /* Initialize variable arguments. */
	for (VARIANT *arg = Arg; arg && argc < 50; arg = va_arg( marker, VARIANT*)) {
		if (!toJSVal(*arg, argv[argc++])) {
			wError_.SetError("Unknown argument type");
			std::strstream str;
			str << "Unknown argument type for argument " << argc << std::ends;
			wError_.SetError(str.str());
			delete str.str();
			return E_FAIL;
		}
		/*
		*/
	}
	va_end( marker );              /* Reset variable arguments.      */
// 		int retcode = fromJSVal(rval, *Ret);
//		return S_OK;

	_bstr_t bstr(Func);
	char *f = bstr;
	jsval	xv=0;
	if (!js_.LookupProperty(f, xv)) {
		std::strstream str;
		str << "Could not find function '" << f << "'" << std::ends;
		wError_.SetError(str.str());
		delete str.str();
		return E_FAIL;
	}

   if (js_.Call(f, argc, argv, &rval)) {
		int retcode = fromJSVal(rval, *Ret);
		return S_OK;
	} else {
		std::strstream str;
		str << "Error calling function '" << f << "'" << std::endl
			<<js_.getError() << std::ends;
		wError_.SetError(str.str());
		delete str.str();
		return E_FAIL;
	}
}

STDMETHODIMP CJSHandler::Call1(BSTR Func, VARIANT Arg1, VARIANT *Ret)
{
	return Call(Func, Ret, &Arg1, NULL);
}

STDMETHODIMP CJSHandler::Call2(BSTR Func, VARIANT Arg1, VARIANT Arg2, VARIANT *Ret)
{
	return Call(Func, Ret, &Arg1, &Arg2, NULL);
}

STDMETHODIMP CJSHandler::Call0(BSTR Func, VARIANT *Ret)
{
	return Call(Func, Ret, NULL);
}

STDMETHODIMP CJSHandler::InstallScript(BSTR Script)
{
	// TODO: Add your implementation code here
	_bstr_t bstr(Script);
	char *script = bstr;

    if (js_.InstallScript(script)) {
		return S_OK;
	} else {
		std::strstream str;
		str << "Error installing script: " << std::endl
			<< js_.getError() << std::ends;
		wError_.SetError(str.str());
		delete str.str();
		return E_FAIL;
	}
}
