/************************************************************
JavaScript Encapsulation Library.

Copyright (C) 2002, Alben Software, Inc.
All rights reserved.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

Alben Software, Inc.
Benny Pollak, President.
20 E 9th Street.
NY, NY 10003
benny@alben.com
*********************************************************************/
// JSScriptXL.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include <strstream>
#include "JSScriptXL.h"
#include "JS.h"

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}


// This is an example of an exported variable
JSSCRIPTXL_API int nJSScriptXL=0;

// This is an example of an exported function.
JSSCRIPTXL_API int fnJSScriptXL()
{
	return 42;
}

// This is the constructor of a class that has been exported.
// see JSScriptXL.h for the class definition
CJSScriptXL::CJSScriptXL()
{ 
	return; 
}

#define USE_EXCEPTIONS

#define DllExport __declspec( dllexport ) 

//#include <afxdisp.h>
#include <comdef.h>
//#include "swaps.h"
#include <atlbase.h>
extern CComModule _Module;
#include <atlcom.h>
#include <exception>
#include "JSCOMUtils.h"

int createArray(VARIANTARG *mvar, double *array); 
VARIANT createVariant(double var);
VARIANT createVariant(const char *s);
//?JSCall@@YG?AUtagVARIANT@@PAU1@  J @Z		long
//?JSCall@@YG?AUtagVARIANT@@PAU1@PAJ @Z		long*
//?JSCall@@YA?AUtagVARIANT@@PAU1@PAJ ZZ		long*,...
//?JSCall@@YA?AUtagVARIANT@@PAU1@    ZZ		...
//?JSCall@@YG?AUtagVARIANT@@PAU1@    @Z
//?JSCall@@YG?AUtagVARIANT@@PAU1@PAJ1@Z		long*,long*
//?JSCall@@YG?AUtagVARIANT@@PAU1@PAJ1@Z
//?JSCall@@YG?AUtagVARIANT@@PAU1@PAJ11@Z	long*,long*,long*
////?JSCall@@YG?AUtagVARIANT@@PAU1@JJJ@Z	long,long,long
extern "C" {
VARIANT DllExport __stdcall JSCall(VARIANTARG *script, VARIANTARG *func, SAFEARRAY **pp)
{
	VARIANT Ret;
    try {
	JS	js_;
	if (!js_.init()) {
		throw exception("Cannot initialize JS engine.");
	}
	if (script->vt != VT_BSTR) {
		throw exception("First argument must be a script");
	}
	if (func->vt != VT_BSTR) {
		throw exception("Second argument must be the name of a function to call");
	}
	_bstr_t sbstr(script);
    if (!js_.InstallScript((char*)sbstr)) {
		std::strstream str;
		str << "Error installing script: " << std::endl
			<< js_.getError() << std::ends;
		Ret = createVariant(str.str());
		delete str.str();
		return Ret;
	}
	jsval		argv[50];
	int			argc = 0;
	SAFEARRAY *pArray = *pp;
	VARIANT *vars = NULL;

	HRESULT hr = SafeArrayAccessData(pArray,(void HUGEP**)&vars);
	unsigned long count = pArray->rgsabound[0].cElements;
	for (int i = 0; i < count; i++) {
		if (!JSCOMUtils::toJSVal(js_, vars[i], argv[argc++])) {
			std::strstream str;
			str << "Unknown argument type for argument " << argc << std::ends;
			exception e(str.str());
			delete str.str();
			SafeArrayUnaccessData(pArray);
			throw e;
		}
	}
	SafeArrayUnaccessData(pArray);

    jsval rval;
	_bstr_t bstr(func);
	char *f = bstr;
	jsval	xv=0;
	if (!js_.LookupProperty(f, xv)) {
		std::strstream str;
		str << "Could not find function '" << f << "'" << std::ends;
		exception e(str.str());
		delete str.str();
		throw e;
	}

    if (js_.Call(f, argc, argv, &rval)) {
		int retcode = JSCOMUtils::fromJSVal(js_, rval, Ret);
		return Ret;
	} else {
		std::strstream str;
		str << "Error calling function '" << f << "'" << std::endl
			<<js_.getError() << std::ends;
		exception e(str.str());
		delete str.str();
		throw e;
	}
    } catch (std::exception& e) {
		const char *cp = e.what();
        return createVariant(cp);
    } catch (...) {
        return createVariant("unknown error");
    }
}
};

//
VARIANT createVariant(const char *s)
{
	VARIANT ret;
	USES_CONVERSION;
	ret.vt = VT_BSTR;
	ret.bstrVal = ::SysAllocString(A2BSTR(s));
	return ret;
}
VARIANT createVariant(double var)
{
	VARIANT ret;
	ret.vt = VT_R8;
	ret.dblVal = var;
	return ret;
}
int createArray(VARIANTARG *mvar, double *array)
{
	HRESULT result;
	double x = 0.0;
	double y = 0.0;
//	int arraydim = 0;
//	long arrayindex[100];
//	arrayindex[0] = 2;
//	arrayindex[1] = 2;
//	arrayindex[2] = 0;
	if (mvar->vt == VT_DISPATCH)
	{
		DISPID dispid;
		LPOLESTR funcname = L"Value";
		result = mvar->pdispVal->GetIDsOfNames(IID_NULL,&funcname,1,LOCALE_SYSTEM_DEFAULT,&dispid);
		if (result == DISP_E_UNKNOWNNAME) return -1;
		VARIANTARG xv;
		if (result == S_OK)
		{
			DISPPARAMS dispparams = {NULL,NULL,0,0};
			mvar->pdispVal->Invoke(dispid,IID_NULL,LOCALE_SYSTEM_DEFAULT,DISPATCH_PROPERTYGET,
			&dispparams,&xv,NULL,NULL);
		}
		VariantCopyInd(mvar,&xv);
	}	
	
	if ((result = VariantChangeType( mvar, mvar, 0, VT_R8)) == S_OK ) {
		 array[0] = mvar->dblVal;
		 return 1;
	}
	
	
	if (mvar->vt == VT_ARRAY + VT_VARIANT)
	{
		SAFEARRAY *pArray = mvar->parray;
		int dims = pArray->cDims; // rgsabounds, 0 lbound 1 ubound
		if (dims != 2) return -3;
		unsigned long lbound = pArray->rgsabound[0].cElements;
		unsigned long ubound = pArray->rgsabound[1].cElements;
		if (ubound < lbound) return -4;
		HUGEP void *vars;
		HRESULT hr = SafeArrayAccessData(pArray, &vars);
		for (unsigned long i = 0; i < ubound-lbound+1; i++) {
			VARIANTARG va = ((VARIANTARG*)vars)[i];
			if ((result = VariantChangeType(&va, &va, 0, VT_R8))==S_OK) {
				array[i] = va.dblVal;
			} else if ((result = VariantChangeType(&va, &va, 0, VT_I4))==S_OK) {
				array[i] = (double)va.lVal;
			} else {
				SafeArrayUnaccessData(pArray);
				return -5;
			}
		}
		SafeArrayUnaccessData(pArray);
		return ubound-lbound+1;
	}
	return -6;
}
